@@ -16,11 +16,22 @@ void MDIO_BeginTimePeriod(const int32 timestamp_base);
 void MDIO_EndTimePeriod(const int32 master_timestamp);
 
 void MDIO_Init(bool overseas, bool PAL, bool overseas_reported, bool PAL_reported);
+void MDIO_Kill(void);
 
 void MDINPUT_Frame(void);
-void MDINPUT_SetInput(int port, const char *type, void *ptr);
+void MDINPUT_SetInput(unsigned port, const char *type, uint8 *ptr);
 
-extern InputInfoStruct MDInputInfo;
+enum
+{
+ MTAP_NONE = 0,
+ MTAP_TP_PRT1 = 1,
+ MTAP_TP_PRT2 = 2,
+ MTAP_TP_DUAL = 3,
+ MTAP_4WAY = 4,
+};
+void MDINPUT_SetMultitap(unsigned type);
+
+extern const std::vector<InputPortInfoStruct> MDPortInfo;
 
 class MD_Input_Device
 {
@@ -36,10 +47,10 @@ class MD_Input_Device
         virtual void UpdatePhysicalState(const void *data);
 	virtual void BeginTimePeriod(const int32 timestamp_base);
 	virtual void EndTimePeriod(const int32 master_timestamp);
-	virtual int StateAction(StateMem *sm, int load, int data_only, const char *section_prefix);
+	virtual void StateAction(StateMem *sm, const unsigned load, const bool data_only, const char *section_prefix);
 };
 
-int MDINPUT_StateAction(StateMem *sm, int load, int data_only);
+void MDINPUT_StateAction(StateMem *sm, const unsigned load, const bool data_only);
 
 }
 
