@@ -1,5 +1,10 @@
-#ifndef __NES_SOUND_H
-#define __NES_SOUND_H
+#ifndef __MDFN_NES_SOUND_H
+#define __MDFN_NES_SOUND_H
+
+#include <vector>
+
+namespace MDFN_IEN_NES
+{
 
 typedef struct __EXPSOUND {
 	   void (*HiFill)(void);
@@ -8,13 +13,11 @@ typedef struct __EXPSOUND {
 	   void (*Kill)(void);
 } EXPSOUND;
 
-#include <vector>
-
 extern std::vector<EXPSOUND> GameExpSound;
 
 int FlushEmulateSound(int reverse, int16 *SoundBuf, int32 MaxSoundFrames);
 
-extern MDFN_ALIGN(16) int16 WaveHiEx[40000];
+alignas(16) extern int16 WaveHiEx[40000];
 
 extern uint32 soundtsoffs;
 #define SOUNDTS (timestamp + soundtsoffs)
@@ -23,13 +26,13 @@ int MDFNSND_Init(bool IsPAL) MDFN_COLD;
 void MDFNSND_Close(void) MDFN_COLD;
 void MDFNSND_Power(void) MDFN_COLD;
 void MDFNSND_Reset(void) MDFN_COLD;
-void MDFNSND_SaveState(void);
-void MDFNSND_LoadState(int version);
 
-void MDFN_SoundCPUHook(int);
-int MDFNSND_StateAction(StateMem *sm, int load, int data_only);
+void MDFN_FASTCALL MDFN_SoundCPUHook(int);
+void MDFNSND_StateAction(StateMem *sm, const unsigned load, const bool data_only);
 void MDFNNES_SetSoundVolume(uint32 volume) MDFN_COLD;
 void MDFNNES_SetSoundMultiplier(double multiplier) MDFN_COLD;
 bool MDFNNES_SetSoundRate(double Rate) MDFN_COLD;
 
+}
+
 #endif
