@@ -52,11 +52,11 @@
 //---------------------------------------------------------------------------
 */
 
-#include "neopop.h"
+#include "../neopop.h"
 #include "TLCS900h_registers.h"
-#include "interrupt.h"
-#include "mem.h"
-#include "bios.h"
+#include "../interrupt.h"
+#include "../mem.h"
+#include "../bios.h"
 #include "TLCS900h_interpret.h"
 #include "TLCS900h_interpret_single.h"
 #include "TLCS900h_interpret_src.h"
