@@ -48,6 +48,21 @@ PS_CPU::PS_CPU()
 
  CPUHook = NULL;
  ADDBT = NULL;
+
+ GTE_Init();
+
+ for(unsigned i = 0; i < 24; i++)
+ {
+  uint8 v = 7;
+
+  if(i < 12)
+   v += 4;
+
+  if(i < 21)
+   v += 3;
+
+  MULT_Tab24[i] = v;
+ }
 }
 
 PS_CPU::~PS_CPU()
@@ -133,7 +148,7 @@ void PS_CPU::Power(void)
  GTE_Power();
 }
 
-int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
+void PS_CPU::StateAction(StateMem *sm, const unsigned load, const bool data_only)
 {
  SFORMAT StateRegs[] =
  {
@@ -161,7 +176,7 @@ int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
   SFARRAY32(CP0.Regs, 32),
 
   SFARRAY(ReadAbsorb, 0x20),
-  SFVAR(ReadAbsorbDummy),
+  SFVARN(ReadAbsorb[0x20], "ReadAbsorbDummy"),
   SFVAR(ReadAbsorbWhich),
   SFVAR(ReadFudge),
 
@@ -169,21 +184,19 @@ int PS_CPU::StateAction(StateMem *sm, int load, int data_only)
 
   SFEND
  };
- int ret = MDFNSS_StateAction(sm, load, data_only, StateRegs, "CPU");
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "CPU");
 
- ret &= GTE_StateAction(sm, load, data_only);
+ GTE_StateAction(sm, load, data_only);
 
  if(load)
  {
 
  }
-
- return(ret);
 }
 
-void PS_CPU::AssertIRQ(int which, bool asserted)
+void PS_CPU::AssertIRQ(unsigned which, bool asserted)
 {
- assert(which >= 0 && which <= 5);
+ assert(which <= 5);
 
  CP0.CAUSE &= ~(1 << (10 + which));
 
@@ -245,6 +258,22 @@ INLINE T PS_CPU::PeekMemory(uint32 address)
  return(ret);
 }
 
+template<typename T>
+void PS_CPU::PokeMemory(uint32 address, T value)
+{
+ address &= addr_mask[address >> 29];
+
+ if(address >= 0x1F800000 && address <= 0x1F8003FF)
+  return ScratchRAM.Write<T>(address & 0x3FF, value);
+
+ if(sizeof(T) == 1)
+  PSX_MemPoke8(address, value);
+ else if(sizeof(T) == 2)
+  PSX_MemPoke16(address, value);
+ else
+  PSX_MemPoke32(address, value);
+}
+
 template<typename T>
 INLINE T PS_CPU::ReadMemory(pscpu_timestamp_t &timestamp, uint32 address, bool DS24, bool LWC_timing)
 {
@@ -429,7 +458,7 @@ uint32 PS_CPU::Exception(uint32 code, uint32 PC, const uint32 NPM)
 #define GPR_RES(n) { unsigned tn = (n); ReadAbsorb[tn] = 0; }
 #define GPR_DEPRES_END ReadAbsorb[0] = back; }
 
-template<bool DebugMode, bool ILHMode>
+template<bool DebugMode, bool BIOSPrintMode, bool ILHMode>
 pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 {
  register pscpu_timestamp_t timestamp = timestamp_in;
@@ -475,17 +504,28 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
     BACKING_TO_ACTIVE;
    }
 
-   if(!ILHMode)
+   if(BIOSPrintMode)
    {
     if(PC == 0xB0)
     {
      if(MDFN_UNLIKELY(GPR[9] == 0x3D))
      {
-      PSX_DBG(PSX_DBG_BIOS_PRINT, "%c", GPR[4]);
+      PSX_DBG_BIOS_PUTC(GPR[4]);
      }
     }
    }
 
+   // We can't fold this into the ICache[] != PC handling, since the lower 2 bits of TV
+   // are already used for cache management purposes and it assumes that the lower 2 bits of PC will be 0.
+   if(MDFN_UNLIKELY(PC & 0x3))
+   {
+    // This will block interrupt processing, but since we're going more for keeping broken homebrew/hacks from working
+    // than super-duper-accurate pipeline emulation, it shouldn't be a problem.
+    new_PC = Exception(EXCEPTION_ADEL, PC, new_PC_mask);
+    new_PC_mask = 0;
+    goto OpDone;
+   }
+
    instr = ICache[(PC & 0xFFC) >> 2].Data;
 
    if(ICache[(PC & 0xFFC) >> 2].TV != PC)
@@ -499,13 +539,13 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
     // FIXME: Handle executing out of scratchpad.
     if(PC >= 0xA0000000 || !(BIU & 0x800))
     {
-     instr = LoadU32_LE((uint32 *)&FastMap[PC >> FAST_MAP_SHIFT][PC]);
+     instr = MDFN_de32lsb<true>(&FastMap[PC >> FAST_MAP_SHIFT][PC]);
      timestamp += 4;	// Approximate best-case cache-disabled time, per PS1 tests(executing out of 0xA0000000+); it can be 5 in *some* sequences of code(like a lot of sequential "nop"s, probably other simple instructions too).
     }
     else
     {
      __ICache *ICI = &ICache[((PC & 0xFF0) >> 2)];
-     const uint32 *FMP = (uint32 *)&FastMap[(PC &~ 0xF) >> FAST_MAP_SHIFT][PC &~ 0xF];
+     const uint8 *FMP = &FastMap[(PC &~ 0xF) >> FAST_MAP_SHIFT][PC &~ 0xF];
 
      // | 0x2 to simulate (in)validity bits.
      ICI[0x00].TV = (PC &~ 0xF) | 0x00 | 0x2;
@@ -520,19 +560,19 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
       case 0x0:
 	timestamp++;
         ICI[0x00].TV &= ~0x2;
-	ICI[0x00].Data = LoadU32_LE(&FMP[0]);
+	ICI[0x00].Data = MDFN_de32lsb<true>(&FMP[0x0]);
       case 0x4:
 	timestamp++;
         ICI[0x01].TV &= ~0x2;
-	ICI[0x01].Data = LoadU32_LE(&FMP[1]);
+	ICI[0x01].Data = MDFN_de32lsb<true>(&FMP[0x4]);
       case 0x8:
 	timestamp++;
         ICI[0x02].TV &= ~0x2;
-	ICI[0x02].Data = LoadU32_LE(&FMP[2]);
+	ICI[0x02].Data = MDFN_de32lsb<true>(&FMP[0x8]);
       case 0xC:
 	timestamp++;
         ICI[0x03].TV &= ~0x2;
-	ICI[0x03].Data = LoadU32_LE(&FMP[3]);
+	ICI[0x03].Data = MDFN_de32lsb<true>(&FMP[0xC]);
 	break;
      }
      instr = ICache[(PC & 0xFFC) >> 2].Data;
@@ -587,7 +627,7 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 	  PC = (PC & new_PC_mask) + new_PC;				\
 	  if(old_PC == ((PC & (mask)) + (offset)))			\
 	  {								\
-	   if(*(uint32 *)&FastMap[PC >> FAST_MAP_SHIFT][PC] == 0)	\
+	   if(MDFN_densb<uint32, true>(&FastMap[PC >> FAST_MAP_SHIFT][PC]) == 0)	\
 	   {								\
 	    if(next_event_ts > timestamp) /* Necessary since next_event_ts might be set to something like "0" to force a call to the event handler. */		\
 	    {								\
@@ -609,53 +649,63 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 	 goto SkipNPCStuff;				\
 	}
 
-   #define ITYPE uint32 rs MDFN_NOWARN_UNUSED = (instr >> 21) & 0x1F; uint32 rt MDFN_NOWARN_UNUSED = (instr >> 16) & 0x1F; int32 immediate = (int16)(instr & 0xFFFF); /*printf(" rs=%02x(%08x), rt=%02x(%08x), immediate=(%08x) ", rs, GPR[rs], rt, GPR[rt], immediate);*/
+   #define ITYPE uint32 rs MDFN_NOWARN_UNUSED = (instr >> 21) & 0x1F; uint32 rt MDFN_NOWARN_UNUSED = (instr >> 16) & 0x1F; uint32 immediate = (int32)(int16)(instr & 0xFFFF); /*printf(" rs=%02x(%08x), rt=%02x(%08x), immediate=(%08x) ", rs, GPR[rs], rt, GPR[rt], immediate);*/
    #define ITYPE_ZE uint32 rs MDFN_NOWARN_UNUSED = (instr >> 21) & 0x1F; uint32 rt MDFN_NOWARN_UNUSED = (instr >> 16) & 0x1F; uint32 immediate = instr & 0xFFFF; /*printf(" rs=%02x(%08x), rt=%02x(%08x), immediate=(%08x) ", rs, GPR[rs], rt, GPR[rt], immediate);*/
    #define JTYPE uint32 target = instr & ((1 << 26) - 1); /*printf(" target=(%08x) ", target);*/
    #define RTYPE uint32 rs MDFN_NOWARN_UNUSED = (instr >> 21) & 0x1F; uint32 rt MDFN_NOWARN_UNUSED = (instr >> 16) & 0x1F; uint32 rd MDFN_NOWARN_UNUSED = (instr >> 11) & 0x1F; uint32 shamt MDFN_NOWARN_UNUSED = (instr >> 6) & 0x1F; /*printf(" rs=%02x(%08x), rt=%02x(%08x), rd=%02x(%08x) ", rs, GPR[rs], rt, GPR[rt], rd, GPR[rd]);*/
 
-   static const void *const op_goto_table[256] =
-   {
-    &&op_SLL, &&op_ILL, &&op_SRL, &&op_SRA, &&op_SLLV, &&op_ILL, &&op_SRLV, &&op_SRAV,
-    &&op_JR, &&op_JALR, &&op_ILL, &&op_ILL, &&op_SYSCALL, &&op_BREAK, &&op_ILL, &&op_ILL,
-    &&op_MFHI, &&op_MTHI, &&op_MFLO, &&op_MTLO, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_MULT, &&op_MULTU, &&op_DIV, &&op_DIVU, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_ADD, &&op_ADDU, &&op_SUB, &&op_SUBU, &&op_AND, &&op_OR, &&op_XOR, &&op_NOR,
-    &&op_ILL, &&op_ILL, &&op_SLT, &&op_SLTU, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
+#if HAVE_COMPUTED_GOTO
+   #define CGBEGIN static const void *const op_goto_table[256] = {
+   #define CGE(l) &&l,
+   #define CGEND }; goto *op_goto_table[opf];
+#else
+   /* (uint8) cast for cheaper alternative to generated branch+compare bounds check instructions, but still more
+      expensive than computed goto which needs no masking nor bounds checking.
+   */
+   #define CGBEGIN { enum { CGESB = 1 + __COUNTER__ }; switch((uint8)opf) {
+   #define CGE(l) case __COUNTER__ - CGESB: goto l;
+   #define CGEND } }
+#endif
 
-    NULL, &&op_BCOND, &&op_J, &&op_JAL, &&op_BEQ, &&op_BNE, &&op_BLEZ, &&op_BGTZ,
-    &&op_ADDI, &&op_ADDIU, &&op_SLTI, &&op_SLTIU, &&op_ANDI, &&op_ORI, &&op_XORI, &&op_LUI,
-    &&op_COP0, &&op_COP1, &&op_COP2, &&op_COP3, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_LB, &&op_LH, &&op_LWL, &&op_LW, &&op_LBU, &&op_LHU, &&op_LWR, &&op_ILL,
-    &&op_SB, &&op_SH, &&op_SWL, &&op_SW, &&op_ILL, &&op_ILL, &&op_SWR, &&op_ILL,
-    &&op_LWC0, &&op_LWC1, &&op_LWC2, &&op_LWC3, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
-    &&op_SWC0, &&op_SWC1, &&op_SWC2, &&op_SWC3, &&op_ILL, &&op_ILL, &&op_ILL, &&op_ILL,
+   CGBEGIN
+    CGE(op_SLL)  CGE(op_ILL)   CGE(op_SRL)  CGE(op_SRA)   CGE(op_SLLV)    CGE(op_ILL)   CGE(op_SRLV) CGE(op_SRAV)
+    CGE(op_JR)   CGE(op_JALR)  CGE(op_ILL)  CGE(op_ILL)   CGE(op_SYSCALL) CGE(op_BREAK) CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_MFHI) CGE(op_MTHI)  CGE(op_MFLO) CGE(op_MTLO)  CGE(op_ILL)     CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_MULT) CGE(op_MULTU) CGE(op_DIV)  CGE(op_DIVU)  CGE(op_ILL)     CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_ADD)  CGE(op_ADDU)  CGE(op_SUB)  CGE(op_SUBU)  CGE(op_AND)     CGE(op_OR)    CGE(op_XOR)  CGE(op_NOR)
+    CGE(op_ILL)  CGE(op_ILL)   CGE(op_SLT)  CGE(op_SLTU)  CGE(op_ILL)     CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)     CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)     CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)
+
+    CGE(op_ILL)  CGE(op_BCOND) CGE(op_J)    CGE(op_JAL)   CGE(op_BEQ)  CGE(op_BNE) CGE(op_BLEZ) CGE(op_BGTZ)
+    CGE(op_ADDI) CGE(op_ADDIU) CGE(op_SLTI) CGE(op_SLTIU) CGE(op_ANDI) CGE(op_ORI) CGE(op_XORI) CGE(op_LUI)
+    CGE(op_COP0) CGE(op_COP1)  CGE(op_COP2) CGE(op_COP3)  CGE(op_ILL)  CGE(op_ILL) CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL)   CGE(op_ILL)  CGE(op_ILL) CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_LB)   CGE(op_LH)    CGE(op_LWL)  CGE(op_LW)    CGE(op_LBU)  CGE(op_LHU) CGE(op_LWR)  CGE(op_ILL)
+    CGE(op_SB)   CGE(op_SH)    CGE(op_SWL)  CGE(op_SW)    CGE(op_ILL)  CGE(op_ILL) CGE(op_SWR)  CGE(op_ILL)
+    CGE(op_LWC0) CGE(op_LWC1)  CGE(op_LWC2) CGE(op_LWC3)  CGE(op_ILL)  CGE(op_ILL) CGE(op_ILL)  CGE(op_ILL)
+    CGE(op_SWC0) CGE(op_SWC1)  CGE(op_SWC2) CGE(op_SWC3)  CGE(op_ILL)  CGE(op_ILL) CGE(op_ILL)  CGE(op_ILL)
 
     // Interrupt portion of this table is constructed so that an interrupt won't be taken when the PC is pointing to a GTE instruction,
     // to avoid problems caused by pipeline vs coprocessor nuances that aren't emulated.
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
 
-    NULL, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_COP2, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-    &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT, &&op_INTERRUPT,
-   };
-
-   goto *op_goto_table[opf];
+    CGE(op_ILL)       CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_COP2)      CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+    CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT) CGE(op_INTERRUPT)
+   CGEND
 
    {
     BEGIN_OPF(ILL, 0, 0);
@@ -822,7 +872,7 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
     BEGIN_OPF(BCOND, 0x01, 0);
 	const uint32 tv = GPR[(instr >> 21) & 0x1F];
 	uint32 riv = (instr >> 16) & 0x1F;
-	int32 immediate = (int16)(instr & 0xFFFF);
+	uint32 immediate = (int32)(int16)(instr & 0xFFFF);
 	bool result = (int32)(tv ^ (riv << 31)) < 0;
 
 	GPR_DEPRES_BEGIN
@@ -1109,7 +1159,8 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 		}
 		break;
 
-	 case 0x10 ... 0x1F:
+	 case 0x10: case 0x11: case 0x12: case 0x13: case 0x14: case 0x15: case 0x16: case 0x17:
+	 case 0x18: case 0x19: case 0x1A: case 0x1B: case 0x1C: case 0x1D: case 0x1E: case 0x1F:
 		//printf("%08x\n", PC);
 	        if(timestamp < gte_ts_done)
 	         timestamp = gte_ts_done;
@@ -1488,8 +1539,7 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 	uint64 result;
 
 	result = (int64)(int32)GPR[rs] * (int32)GPR[rt];
-	muldiv_ts_done = timestamp + 7;
-
+	muldiv_ts_done = timestamp + MULT_Tab24[MDFN_lzcount32((GPR[rs] ^ ((int32)GPR[rs] >> 31)) | 0x400)];
 	DO_LDS();
 
 	LO = result;
@@ -1511,8 +1561,7 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 	uint64 result;
 
 	result = (uint64)GPR[rs] * GPR[rt];
-	muldiv_ts_done = timestamp + 7;
-
+	muldiv_ts_done = timestamp + MULT_Tab24[MDFN_lzcount32(GPR[rs] | 0x400)];
 	DO_LDS();
 
 	LO = result;
@@ -1654,7 +1703,7 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
 	GPR_RES(rt);
 	GPR_DEPRES_END
 
-	uint32 result = (bool)((int32)GPR[rs] < immediate);
+	uint32 result = (bool)((int32)GPR[rs] < (int32)immediate);
 
 	DO_LDS();
 
@@ -2256,16 +2305,21 @@ pscpu_timestamp_t PS_CPU::RunReal(pscpu_timestamp_t timestamp_in)
  return(timestamp);
 }
 
-pscpu_timestamp_t PS_CPU::Run(pscpu_timestamp_t timestamp_in, bool ILHMode)
+pscpu_timestamp_t PS_CPU::Run(pscpu_timestamp_t timestamp_in, bool BIOSPrintMode, bool ILHMode)
 {
  if(CPUHook || ADDBT)
-  return(RunReal<true, false>(timestamp_in));
+  return(RunReal<true, true, false>(timestamp_in));
  else
  {
   if(ILHMode)
-   return(RunReal<false, true>(timestamp_in));
+   return(RunReal<false, false, true>(timestamp_in));
   else
-   return(RunReal<false, false>(timestamp_in));
+  {
+   if(BIOSPrintMode)
+    return(RunReal<false, true, false>(timestamp_in));
+   else
+    return(RunReal<false, false, false>(timestamp_in));
+  }
  }
 }
 
@@ -2384,6 +2438,20 @@ uint32 PS_CPU::PeekMem32(uint32 A)
  return PeekMemory<uint32>(A);
 }
 
+void PS_CPU::PokeMem8(uint32 A, uint8 V)
+{
+ PokeMemory<uint8>(A, V);
+}
+
+void PS_CPU::PokeMem16(uint32 A, uint16 V)
+{
+ PokeMemory<uint16>(A, V);
+}
+
+void PS_CPU::PokeMem32(uint32 A, uint32 V)
+{
+ PokeMemory<uint32>(A, V);
+}
 
 #undef BEGIN_OPF
 #undef END_OPF
