@@ -49,8 +49,8 @@ class PS_CPU
 {
  public:
 
- PS_CPU();
- ~PS_CPU();
+ PS_CPU() MDFN_COLD;
+ ~PS_CPU() MDFN_COLD;
 
  // FAST_MAP_* enums are in BYTES(8-bit), not in 32-bit units("words" in MIPS context), but the sizes
  // will always be multiples of 4.
@@ -64,12 +64,12 @@ class PS_CPU
   next_event_ts = next_event_ts_arg;
  }
 
- pscpu_timestamp_t Run(pscpu_timestamp_t timestamp_in, bool ILHMode);
+ pscpu_timestamp_t Run(pscpu_timestamp_t timestamp_in, bool BIOSPrintMode, bool ILHMode);
 
- void Power(void);
+ void Power(void) MDFN_COLD;
 
  // which ranges 0-5, inclusive
- void AssertIRQ(int which, bool asserted);
+ void AssertIRQ(unsigned which, bool asserted);
 
  void SetHalt(bool status);
 
@@ -77,15 +77,12 @@ class PS_CPU
  void SetBIU(uint32 val);
  uint32 GetBIU(void);
 
- int StateAction(StateMem *sm, int load, int data_only);
+ void StateAction(StateMem *sm, const unsigned load, const bool data_only);
 
  private:
 
- struct
- {
-  uint32 GPR[32];
-  uint32 GPR_dummy;	// Used in load delay simulation(indexing past the end of GPR)
- };
+ uint32 GPR[32 + 1];	// GPR[32] Used as dummy in load delay simulation(indexing past the end of real GPR)
+
  uint32 LO;
  uint32 HI;
 
@@ -179,19 +176,16 @@ class PS_CPU
  //WrAbsorbShift += 8;
 #endif
 
- struct
- {
-  uint8 ReadAbsorb[0x20];
-  uint8 ReadAbsorbDummy;
- };
+ uint8 ReadAbsorb[0x20 + 1];
  uint8 ReadAbsorbWhich;
  uint8 ReadFudge;
 
  //uint32 WriteAbsorb;
  //uint8 WriteAbsorbCount;
  //uint8 WriteAbsorbMonkey;
+ uint8 MULT_Tab24[24];
 
- MultiAccessSizeMem<1024, uint32, false> ScratchRAM;
+ MultiAccessSizeMem<1024, false> ScratchRAM;
 
  //PS_GTE GTE;
 
@@ -217,9 +211,10 @@ class PS_CPU
 
  uint32 Exception(uint32 code, uint32 PC, const uint32 NPM) MDFN_WARN_UNUSED_RESULT;
 
- template<bool DebugMode, bool ILHMode> pscpu_timestamp_t RunReal(pscpu_timestamp_t timestamp_in);
+ template<bool DebugMode, bool BIOSPrintMode, bool ILHMode> pscpu_timestamp_t RunReal(pscpu_timestamp_t timestamp_in) NO_INLINE;
 
  template<typename T> T PeekMemory(uint32 address) MDFN_COLD;
+ template<typename T> void PokeMemory(uint32 address, T value) MDFN_COLD;
  template<typename T> T ReadMemory(pscpu_timestamp_t &timestamp, uint32 address, bool DS24 = false, bool LWC_timing = false);
  template<typename T> void WriteMemory(pscpu_timestamp_t &timestamp, uint32 address, uint32 value, bool DS24 = false);
 
@@ -247,9 +242,15 @@ class PS_CPU
  uint32 GetRegister(unsigned int which, char *special, const uint32 special_len);
  void SetRegister(unsigned int which, uint32 value);
  bool PeekCheckICache(uint32 PC, uint32 *iw);
+
  uint8 PeekMem8(uint32 A);
  uint16 PeekMem16(uint32 A);
  uint32 PeekMem32(uint32 A);
+
+ void PokeMem8(uint32 A, uint8 V);
+ void PokeMem16(uint32 A, uint16 V);
+ void PokeMem32(uint32 A, uint32 V);
+
  private:
  void (*CPUHook)(const pscpu_timestamp_t timestamp, uint32 pc);
  void (*ADDBT)(uint32 from, uint32 to, bool exception);
