@@ -2,13 +2,13 @@
 #define __MDFN_PSX_PSX_H
 
 #include <mednafen/mednafen.h>
-#include <mednafen/masmem.h>
 #include <mednafen/cdrom/cdromif.h>
 #include <mednafen/general.h>
 #include <mednafen/FileStream.h>
 
 #include <trio/trio.h>
 
+#include "masmem.h"
 
 //
 // Comment out these 2 defines for extra speeeeed.
@@ -35,12 +35,14 @@ namespace MDFN_IEN_PSX
  #define PSX_DBG_FLOOD		4	// Heavy informational debug messages(GPU commands; TODO).
 
 #if PSX_DBGPRINT_ENABLE
- void PSX_DBG(unsigned level, const char *format, ...) throw() MDFN_COLD MDFN_FORMATSTR(gnu_printf, 2, 3);
+ void PSX_DBG(unsigned level, const char *format, ...) noexcept MDFN_COLD MDFN_FORMATSTR(gnu_printf, 2, 3);
+ void PSX_DBG_BIOS_PUTC(uint8 c) noexcept;
 
  #define PSX_WARNING(format, ...) { PSX_DBG(PSX_DBG_WARNING, format "\n", ## __VA_ARGS__); }
  #define PSX_DBGINFO(format, ...) { }
 #else
  static INLINE void PSX_DBG(unsigned level, const char* format, ...) { }
+ static INLINE void PSX_DBG_BIOS_PUTC(uint8 c) { }
  static INLINE void PSX_WARNING(const char* format, ...) { }
  static INLINE void PSX_DBGINFO(const char* format, ...) { }
 #endif
@@ -64,11 +66,9 @@ namespace MDFN_IEN_PSX
  uint32 PSX_MemPeek32(uint32 A);
 
  // Should write to WO-locations if possible
- #if 0
  void PSX_MemPoke8(uint32 A, uint8 V);
  void PSX_MemPoke16(uint32 A, uint16 V);
  void PSX_MemPoke32(uint32 A, uint32 V);
- #endif
 
  void PSX_RequestMLExit(void);
  void ForceEventUpdates(const pscpu_timestamp_t timestamp);
@@ -89,6 +89,8 @@ namespace MDFN_IEN_PSX
  #define PSX_EVENT_MAXTS       		0x20000000
  void PSX_SetEventNT(const int type, const pscpu_timestamp_t next_timestamp);
 
+ void PSX_SetDMACycleSteal(unsigned stealage);
+
  void PSX_GPULineHook(const pscpu_timestamp_t timestamp, const pscpu_timestamp_t line_timestamp, bool vsync, uint32 *pixels, const MDFN_PixelFormat* const format, const unsigned width, const unsigned pix_clock_offset, const unsigned pix_clock, const unsigned pix_clock_divider);
 
  uint32 PSX_GetRandU32(uint32 mina, uint32 maxa);
@@ -112,7 +114,7 @@ namespace MDFN_IEN_PSX
  extern PS_GPU *GPU;
  extern PS_CDC *CDC;
  extern PS_SPU *SPU;
- extern MultiAccessSizeMem<2048 * 1024, uint32, false> MainRAM;
+ extern MultiAccessSizeMem<2048 * 1024, false> MainRAM;
 };
 
 
