@@ -24,11 +24,6 @@
 #include <time.h>
 #include <zlib.h>
 
-void QTRecord::w8(uint8 val)
-{
- qtfile.put_char(val);
-}
-
 void QTRecord::w16(uint16 val)
 {
  uint8 buf[2];
@@ -91,32 +86,18 @@ void QTRecord::w64(uint64 val)
  qtfile.write(buf, sizeof(buf));
 }
 
-// max_len doesn't include the leading 1-byte count
-// returns number of bytes written.
-uint32 QTRecord::wps(const char *str, uint32 max_len)
+// fixed_len doesn't include the leading 1-byte count
+void QTRecord::wps(const char *str, uint8 fixed_len)
 {
- if(!max_len)	// Variable-size pascal string
- {
-  max_len = strlen(str);
-  if(max_len > 255)
-   max_len = 255;
- }
-
- uint32 count = strlen(str);
- char buf[1 + max_len];
+ uint8 slen = std::min<size_t>(255, strlen(str));
+ uint8 count = (fixed_len ? fixed_len : slen);
+ char buf[1 + 255];
 
  memset(buf, 0, sizeof(buf));
-
- if(count > max_len)
-  count = max_len;
-
- strncpy(buf + 1, str, count);
-
  buf[0] = count;
+ memcpy(&buf[1], str, std::min<uint8>(slen, count));
 
- qtfile.write(buf, sizeof(buf));
-
- return(sizeof(buf));
+ qtfile.write(buf, 1 + count);
 }
 
 void QTRecord::vardata_begin(void)
@@ -128,8 +109,8 @@ void QTRecord::vardata_begin(void)
 
 void QTRecord::vardata_end(void)
 {
- int64 cur_offset = qtfile.tell();
- int64 start_offset = vardata_foffsets.back();
+ uint64 cur_offset = qtfile.tell();
+ uint64 start_offset = vardata_foffsets.back();
 
  vardata_foffsets.pop_back();
 
@@ -178,8 +159,8 @@ void QTRecord::atom_begin(const char *type, bool small_atom)
 
 void QTRecord::atom_end(void)
 {
- int64 cur_offset = qtfile.tell();
- int64 start_offset = atom_foffsets.back();
+ uint64 cur_offset = qtfile.tell();
+ uint64 start_offset = atom_foffsets.back();
  bool small_atom = atom_smalls.back();
 
  atom_foffsets.pop_back();
@@ -200,7 +181,7 @@ void QTRecord::atom_end(void)
  qtfile.seek(cur_offset, SEEK_SET);
 }
 
-QTRecord::QTRecord(const char *path, const VideoSpec &spec) : qtfile(path, FileWrapper::MODE_WRITE_SAFE), resampler(NULL)
+QTRecord::QTRecord(const std::string& path, const VideoSpec &spec) : qtfile(path, FileStream::MODE_WRITE_SAFE), resampler(NULL)
 {
  Finished = false;
 
@@ -311,7 +292,7 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
    int x_start;
    int width;
    int xscale_factor;
-   int32 dest_x;
+   uint32 dest_x;
    uint32 *src_ptr;
    uint8 *dest_line;
 
@@ -414,9 +395,12 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
  {
   static uint8 workmem[LZO1X_1_MEM_COMPRESS];
   lzo_uint dst_len = CompressedVideoBuffer.size();
+  uint8 tmp[2];
 
-  qtfile.put_char((0 << 1) | 0x1);
-  qtfile.put_char(0);
+  tmp[0] = (0 << 1) | 0x1;
+  tmp[1] = 0;
+
+  qtfile.write(tmp, 2);
 
   lzo1x_1_compress(&RawVideoBuffer[0], RawVideoBuffer.size(), &CompressedVideoBuffer[0], &dst_len, workmem);
 
@@ -475,7 +459,7 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
   if((ResampInBuffer.size() - (ResampInBufferFramesInCount * SoundChan)) < SoundBufSize * SoundChan)
    ResampInBuffer.resize((ResampInBufferFramesInCount + SoundBufSize) * SoundChan);
 
-  for(int i = 0; i < SoundBufSize * SoundChan; i++)
+  for(unsigned i = 0; i < SoundBufSize * SoundChan; i++)
    ResampInBuffer[ResampInBufferFramesInCount * SoundChan + i] = SoundBuf[i];
 
   ResampInBufferFramesInCount += SoundBufSize;
@@ -508,7 +492,7 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
   ResampInBufferFramesInCount -= in_len;
   SoundBufROSize = out_len;
 
-  for(int i = 0; i < SoundBufROSize * SoundChan; i++)
+  for(unsigned i = 0; i < SoundBufROSize * SoundChan; i++)
    MDFN_en16msb((uint8 *)&ResampOutBuffer[i], ResampOutBuffer[i]);
  }
  else
@@ -518,7 +502,7 @@ void QTRecord::WriteFrame(const MDFN_Surface *surface, const MDFN_Rect &DisplayR
   if(ResampOutBuffer.size() < SoundBufSize * SoundChan)
    ResampOutBuffer.resize(SoundBufSize * SoundChan);
 
-  for(int i = 0; i < SoundBufROSize * SoundChan; i++)
+  for(unsigned i = 0; i < SoundBufROSize * SoundChan; i++)
    MDFN_en16msb((uint8 *)&ResampOutBuffer[i], SoundBuf[i]);
  }
 
@@ -879,10 +863,10 @@ void QTRecord::Write_stbl(void)
 
  bool need64bit_offset = false;
 
- if(OnAudioTrack && QTChunks.back().audio_foffset >= ((int64)1 << 32))
+ if(OnAudioTrack && QTChunks.back().audio_foffset >= ((uint64)1 << 32))
   need64bit_offset = true;
 
- if(!OnAudioTrack && QTChunks.back().video_foffset >= ((int64)1 << 32))
+ if(!OnAudioTrack && QTChunks.back().video_foffset >= ((uint64)1 << 32))
   need64bit_offset = true;
 
  if(need64bit_offset)
