@@ -46,7 +46,7 @@ unsigned MappedRAM::size() const { return size_; }
 
 uint8 MappedRAM::read(unsigned addr) { return data_[addr]; }
 void MappedRAM::write(unsigned addr, uint8 n) { if(!write_protect_) data_[addr] = n; }
-const uint8 MappedRAM::operator[](unsigned addr) const { return data_[addr]; }
+uint8 MappedRAM::operator[](unsigned addr) const { return data_[addr]; }
 MappedRAM::MappedRAM() : data_(0), size_(-1U), write_protect_(false) {}
 
 //Bus
