@@ -4,15 +4,16 @@
 namespace MDFN_IEN_VB
 {
 
-bool VIP_Init(void) MDFN_COLD;
+void VIP_Init(void) MDFN_COLD;
+void VIP_Kill(void) MDFN_COLD;
 void VIP_Power(void) MDFN_COLD;
 
-void VIP_SetInstantDisplayHack(bool);
-void VIP_SetAllowDrawSkip(bool);
-void VIP_Set3DMode(uint32 mode, bool reverse, uint32 prescale, uint32 sbs_separation);
-void VIP_SetParallaxDisable(bool disabled);
-void VIP_SetDefaultColor(uint32 default_color);
-void VIP_SetAnaglyphColors(uint32 lcolor, uint32 rcolor);	// R << 16, G << 8, B << 0
+void VIP_SetInstantDisplayHack(bool) MDFN_COLD;
+void VIP_SetAllowDrawSkip(bool) MDFN_COLD;
+void VIP_Set3DMode(uint32 mode, bool reverse, uint32 prescale, uint32 sbs_separation) MDFN_COLD;
+void VIP_SetParallaxDisable(bool disabled) MDFN_COLD;
+void VIP_SetDefaultColor(uint32 default_color) MDFN_COLD;
+void VIP_SetAnaglyphColors(uint32 lcolor, uint32 rcolor) MDFN_COLD;	// R << 16, G << 8, B << 0
 
 v810_timestamp_t MDFN_FASTCALL VIP_Update(const v810_timestamp_t timestamp);
 void VIP_ResetTS(void);
@@ -26,9 +27,7 @@ uint16 VIP_Read16(v810_timestamp_t &timestamp, uint32 A);
 void VIP_Write8(v810_timestamp_t &timestamp, uint32 A, uint8 V);
 void VIP_Write16(v810_timestamp_t &timestamp, uint32 A, uint16 V);
 
-
-
-int VIP_StateAction(StateMem *sm, int load, int data_only);
+void VIP_StateAction(StateMem *sm, const unsigned load, const bool data_only);
 
 
 enum
